#! /bin/perl


# analyze.pl: script for carrying out probabilistic program analysis task.
#     Copyright (C) 2013  Sriram Sankaranarayanan <srirams@colorado.edu>
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.




use Time::HiRes qw(time);
#use strict;
use Getopt::Long qw(GetOptions);
use Math::BigFloat;
use Cwd 'abs_path';
use File::Basename;


$depth = 12;
$inputLim=90;
$home = dirname(abs_path($0));
printf "PERL script executing from %s \n", $home;
$resultsDir = "/tmp/run/";
$volComp = $home."/volcomp";
$volCompOptions = sprintf "-d %d", $depth;
$impMain = $home."/frontend";
$impOptions = "";
$outputFile="";
$inputPrg = "example-book-simple";
$tPaths=0.0;
$tWrite=0.0;
$nExec=0;
$nPaths=0;
$tVolComp = 0.0;
$probNaiveEst =Math::BigFloat->new("0.0");
$probBoxEst =Math::BigFloat->new("0.0");
$lb = Math::BigFloat->new("0.0");
$ub = Math::BigFloat->new("0.0");
$propCount = 0;
@propStrs=();
$vcResDir="";


sub getLoggingTime {

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
    my $nice_timestamp = sprintf ( "%04d%02d%02d %02d:%02d:%02d",
                                   $year+1900,$mon+1,$mday,$hour,$min,$sec);
    return $nice_timestamp;
}

sub arrangeProps {
    open PFILE, "<".$inputPrg.".prp" or die ("Cannot open file".$inputPrg.".prp") ;
    $propCount = 0;
    while ($line=<PFILE>){
	chomp($line);
	if ($line =~ m/estimateProb/){
	    $line =~ s/\;//; # remove semicolons
	    printf "Property - %d: %s \n", $propCount, $line;
	    push(@propStrs,$line);
	    $propCount = $propCount +1;
	}
    }
    close PFILE;
}

sub createImpFile {
    my($prpID, $prpString) = @_;
    $cmd1 = sprintf ("cp %s.i  %s/%s-%d.imp",  $inputPrg, $resultsDir, $inputPrg, $prpID);
    printf "> %s \n", $cmd1;
    system($cmd1);
  
    $cmd2 = sprintf ("echo \"%s\" >> %s/%s-%d.imp", $prpString, $resultsDir, $inputPrg, $prpID);
    printf "> %s \n", $cmd2;
    system($cmd2);    
}

sub initRun {
    my($prpID,$prpString)=@_;
    createImpFile($prpID, $prpString);
    $vcResDir = sprintf ("/tmp/run/%s-%d/", $inputPrg,$prpID);
    $outputFile = sprintf "/tmp/run/%s-%d.out", $inputPrg,$prpID;
    $impOptions = sprintf "--dir %s --lim %d < /tmp/run/%s-%d.imp 2> %s ", $vcResDir, $inputLim,$inputPrg,$prpID, $outputFile;

    $tPaths=0.0;
    $tWrite=0.0;
    $nExec=0;
    $nPaths=0;
    $tVolComp = 0.0;
    $probNaiveEst =Math::BigFloat->new("0.0");
    $probBoxEst =Math::BigFloat->new("0.0");
    $lb = Math::BigFloat->new("0.0");
    $ub = Math::BigFloat->new("0.0");
    $cmdToRun = sprintf "mkdir -p %s/%d", $vcResDir, $inputLim;
    system($cmdToRun);

    
}




sub runImp {
    my($prpID,$prpString)=@_;
    $cmdToRun  = $impMain."  ".$impOptions;
    print $cmdToRun;
    system($cmdToRun);
    #$outputFile=$inputPrg.".out";
    open OUTFILE, "<".$outputFile or die $!;
    while ($line = <OUTFILE>){
	if ($line =~ m/Took:\s*(\d*.\d+)\s*to\s*generate\s*paths./){
	    $tPaths = $1;
	    printf "Time to exec paths: %f\n", $tPaths;
	}
	if ($line =~  m/Took:\s*(\d*.\d+)\s*to\s*write\s*paths\s*out./){
	    $tWrite = $1;
	    printf "Time to write paths: %f\n", $tWrite;
	}
	if ($line =~ m/Total\s*Executions:\s*(\d+)\D/){
	    $nExec = $1;
	    printf "#exec: %d\n", $nExec;
	}
	if ($line =~ m/Unique\s*Paths:\s*(\d+)\D/){
	    $nPaths=$1;
	    printf "#unique Paths: %d\n", $nPaths;
	}
    }

    close OUTFILE;
}


sub processFile{
    my($fName0)=@_;
    $fName = sprintf "%s/%d/%s", $vcResDir, $inputLim,$fName0;
    $cmdName = sprintf("grep -v \"inf\" %s > tmpOut ", $fName);
    system($cmdName);
    $cmdName = sprintf("time %s %s   tmpOut > %s.output", $volComp, $volCompOptions,$fName);
    $tStart = time;
    system($cmdName);

    $tEnd = time;
    $tVolComp = $tVolComp + $tEnd - $tStart;
    open RESFILE, "<".$fName.".output" or die $!;
    
    while ($line = <RESFILE>){
	if ($line =~ m/pSimpleEst\s*=\s*(\d*.\d+e?-?\d*)/){
	    printf "Naive estimate: %f \n", $1;
	    $probNaiveEst -> badd(Math::BigFloat -> new($1));
	}

	if ($line =~ m/pBoxEst\s*=\s*(\d*\.\d+e?-?\d*)/){
	    printf "Box estimate: %f \n", $1;
	    $probBoxEst -> badd(Math::BigFloat -> new($1));
	}

	if ($line =~ m/Upper\s*Bound\s*:\s*(\d*.\d+e?-?\d*)/){
	    printf "Upper Bound: %f \n", $1;
	    $ub -> badd( Math::BigFloat -> new ($1));
	    printf "Cumulative upper bound: %.10f \n", $ub-> bround(10);
	}
	if ($line =~ m/Lower\s*Bound\s*:\s*(\d*.\d+e?-?\d*)/){
	    printf "Lower Bound: %f \n", $1;
	    $lb -> badd( Math::BigFloat -> new ($1));
	    printf "Cumulative lower bound: %.10f \n", $lb-> bround(10);

	}

    }
#    close INFILE;
    close RESFILE;
}

sub processProgramAndProperty {
    my ($prgName) = @_;
    $inputPrg = $prgName;
    
    arrangeProps;

    for($p=0; $p <= $#propStrs ; ++$p){
	$propName = $propStrs[$p];
	initRun($p,$propStrs[$p]);

	runImp($p,$propStrs[$p]);
	
	$tVolComp = 0.0;
	$probNaiveEst =Math::BigFloat->new("0.0");
	$probBoxEst =Math::BigFloat->new("0.0");
	$lb = Math::BigFloat->new("0.0");
	$ub = Math::BigFloat->new("0.0");


	for ($i=0; $i < $nPaths; ++$i){
	   
	    $fileName = sprintf("test-%d.in", $i);
	    processFile($fileName);
	}

	printf CSVFILE "%s (paths %s), %d, %d, %.2f, %.2f, %d, %d,", $inputPrg, $propName,$inputLim, $depth, $tPaths,$tWrite,$nExec,$nPaths;
	printf CSVFILE " %.2f, %.10f,  %.10f \n", $tVolComp,  $lb-> bround(10), $ub-> bround(10);

	$pathLB = $lb -> bround(10);

	$tVolComp = 0.0;
	$probNaiveEst =Math::BigFloat->new("0.0");
	$probBoxEst =Math::BigFloat->new("0.0");
	$lb = Math::BigFloat->new("0.0");
	$ub = Math::BigFloat->new("0.0");

	
	for ($i=0; $i < $nPaths; ++$i){
	    $fileName = sprintf("test-%d-phi-0.in", $i);
	    processFile($fileName);
	}
	printf CSVFILE "%s (%s), %d, %d, %.2f, %.2f, %d, %d,", $inputPrg, $propName, $inputLim, $depth, $tPaths,$tWrite,$nExec,$nPaths;
	printf CSVFILE " %.2f, %.10f,   %.10f \n", $tVolComp, $lb-> bround(10) , ($ub-> bround(10) + (1.0 - $pathLB));
    }
}
    

$cmdToRun = sprintf("mkdir -p %s \n", $resultsDir);
system($cmdToRun);


open CSVFILE, ">> results.csv";


@timeData = getLoggingTime();
print CSVFILE " --- RUN STARTED AT : @timeData\n";

printf CSVFILE "Name, inpLim, depth, timePaths, timeWrite, nExec, nUniquePaths, time (VolComp), LB (volComp), UB (volComp) \n";



($ret,$args) = GetOptions('l=i' => \$inputLim,
			  'd=i' => \$depth,
                          '<>' => \&processProgramAndProperty);

@timeData = getLoggingTime();
print CSVFILE " --- RUN ENDED AT : @timeData\n";
