(*--- 

frontend: A prototype frontend for probabilistic programs

    Copyright (C) 2013  Aleksandar Chakarov <alch4231@colorado.edu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

--*)

{
open Parse
let lineNum = ref 1
} 

let blank = [' ' '\012' '\r' '\t']

rule initial = parse
  "/*"  { let _ = comment lexbuf in initial lexbuf }
| "(*"  { let _ = comment2 lexbuf in initial lexbuf }
| "//"  { endline lexbuf }
| blank { initial lexbuf }
| '\n'  { incr lineNum; initial lexbuf }
| '+'           { PLUS }
| '-'           { MINUS }
| '*'           { TIMES }
| '/'           { DIVIDE }
| "true"        { TRUE }
| "false"       { FALSE }
| "flip"        { FLIP }
| "unifReal"    { UNIFREAL }
| "unifInt"     { UNIFINT }
| "gaussian"    { GAUSSIAN }
| "laplacian"   { LAPLACIAN }
| "=="          { EQ_TOK }
| "<="          { LE_TOK }
| "<"           { LT_TOK }
| ">="          { GE_TOK }
| ">"           { GT_TOK }
| '!'           { NOT }
| "&&"          { AND }
| "||"          { OR }
| "skip"        { SKIP }
| "="           { ASSIGN }
| ';'           { SEMICOLON }
| ','           { COMMA }
| "if"          { IF }
| "then"        { THEN }
| "else"        { ELSE }
| "while"       { WHILE }
| "do"          { DO }
| "end"         { END }
| "print"       { PRINT }
| "estimateExp" { EXPECT }
| "estimateProb" { PROB }
| "assume"      { ASSUME }

| '('           { LPAREN }
| ')'           { RPAREN } 
(*| '{'           { LBRACE }
| '}'           { RBRACE } *)

| ("0x")?'-'?['0'-'9']+'.'['0'-'9']* {
  let str = Lexing.lexeme lexbuf in 
  REAL((float_of_string str)) }

| ("0x")?'-'?['0'-'9']+ {
  let str = Lexing.lexeme lexbuf in 
  INT((int_of_string str)) }

| ['A'-'Z''a'-'z''_']['0'-'9''A'-'Z''a'-'z''_']* {
  let str = Lexing.lexeme lexbuf in 
  IDENTIFIER(str)
  } 
| '.' 
| eof     { EOF } 
| _       { 
  Printf.printf "invalid character '%s'\n" (Lexing.lexeme lexbuf) ;
  (* this is not the kind of error handling you want in real life *)
  exit 1 }

and comment = parse
      "*/"  { () }
|     '\n'  { comment lexbuf }
|     eof   { Printf.printf "unterminated /* comment\n" ; exit 1 }
|     _     { comment lexbuf }
and comment2 = parse
      "*)"  { () }
|     '\n'  { comment2 lexbuf }
|     "(*"  { (* ML-style comments can be nested *) 
              let _ = comment2 lexbuf in comment2 lexbuf }
|     eof   { Printf.printf "unterminated (* comment\n" ; exit 1 }
|     _     { comment2 lexbuf }
and endline = parse
        '\n'      { initial lexbuf}
| _               { endline lexbuf}
|       eof       { EOF }
