
%{

(*--- 

frontend: A prototype frontend for probabilistic programs

    Copyright (C) 2013  Aleksandar Chakarov <alch4231@colorado.edu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

--*)


open Imp

let error msg	= failwith msg

%}

%token <string>         IDENTIFIER
%token <int>            INT
%token <float>          REAL

%token PLUS 
%token MINUS 
%token TIMES 
%token DIVIDE
%token TRUE
%token FALSE
%token FLIP
%token UNIFREAL
%token UNIFINT
%token GAUSSIAN
%token LAPLACIAN
%token EQ_TOK
%token LT_TOK
%token LE_TOK
%token GT_TOK
%token GE_TOK
%token NOT
%token AND
%token OR 
%token SKIP
%token ASSIGN 
%token SEMICOLON
%token COMMA
%token IF
%token THEN
%token ELSE
%token WHILE
%token DO
%token END
%token PRINT
%token LPAREN
%token RPAREN
%token LBRACE
%token RBRACE
%token PROB
%token ASSUME
%token EXPECT

%token EOF

%start prog
%type <Imp.prog> prog

%left AND
%left OR
%left PLUS MINUS
%left TIMES DIVIDE
%left LE_TOK LT_TOK GE_TOK GT_TOK EQ_TOK
%nonassoc NOT

%%

aexp : INT                                    { IntConst($1) }
| REAL                                        { RealConst($1) }
| IDENTIFIER                                  { Var($1) }
| aexp PLUS aexp                              { Add ($1,$3) } 
| aexp MINUS aexp                             { Sub ($1,$3) } 
| aexp TIMES aexp                             { Mul ($1,$3) } 
| aexp DIVIDE aexp                            { Div ($1,$3) }
| UNIFREAL LPAREN aexp SEMICOLON aexp RPAREN  { UnifReal ($3,$5) }
| UNIFREAL LPAREN aexp COMMA aexp RPAREN      { UnifReal ($3,$5) }
| UNIFINT LPAREN aexp SEMICOLON aexp RPAREN   { UnifInt ($3,$5) }
| UNIFINT LPAREN aexp COMMA aexp RPAREN       { UnifInt ($3,$5) }
| GAUSSIAN LPAREN aexp SEMICOLON aexp RPAREN  { Gaussian ($3,$5) }
| GAUSSIAN LPAREN aexp COMMA aexp RPAREN      { Gaussian ($3,$5) }
| LAPLACIAN LPAREN aexp SEMICOLON aexp RPAREN { Laplacian ($3,$5) }
| LAPLACIAN LPAREN aexp COMMA aexp RPAREN     { Laplacian ($3,$5) }
| LPAREN aexp RPAREN                          { $2 } 
;

bexp : TRUE                                  { True }
| FALSE                                      { False }
| FLIP aexp                                  { Flip($2) }
| aexp EQ_TOK aexp                           { EQ ($1,$3) }
| aexp LE_TOK aexp                           { LE($1,$3) }
| aexp LT_TOK aexp                           { LT($1,$3) }
| aexp GE_TOK aexp                           { GE($1,$3) }
| aexp GT_TOK aexp                           { GT($1,$3) }
| NOT bexp                                   { Not($2) }
| bexp AND bexp                              { And($1,$3) }
| bexp OR bexp                               { Or($1,$3) }
| LPAREN bexp RPAREN                         { $2 }
;

com : IDENTIFIER ASSIGN aexp                 { Assign(None,$1,$3) } 
| IF bexp THEN com_list END                  { If(None,$2,List.rev $4,None, None) }
| IF bexp THEN com_list ELSE com_list END    { If(None,$2,List.rev $4,Some(List.rev $6), None) }
| WHILE bexp DO com_list END                 { While(None,$2,List.rev $4,None) }
| PRINT aexp                                 { Print(None,$2) }
| PROB bexp                                  { Prob(None,$2) }
| ASSUME bexp                                { Assume(None,$2) }
| EXPECT aexp                                { Exp(None,$2) }
;

com_list : com_list SEMICOLON com            { $3 :: $1 }
| com_list SEMICOLON                         { $1 }
| com                                        { [$1] }
;

prog : com_list                              { List.rev $1 }



