(*--- 

frontend: A prototype frontend for probabilistic programs

    Copyright (C) 2013  Aleksandar Chakarov <alch4231@colorado.edu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

--*)


let limit_runs       = ref 90 (* If no new path within last limit_runs number of executions then terminate. *)
let iteration = ref 0 
let unique_paths_num = ref 0
let path'            = ref []

let path_debug = ref false

let collect_paths (program) (upaths) = 
  let no_new = ref 0 in
  iteration := 0;
  while (!no_new < !limit_runs) do
    
    iteration := !iteration + 1;
    prerr_endline ("Iteration: "^(string_of_int !iteration)); 
    Util.branches := Util.make_branch_hash ();
    let sigma_0 = ConcreteExec.empty_concrete_state in 
    (*let sym_sigma_0 = SymExec.empty_symbolic_state in
    let ssa_0 = SymExec.empty_ssa_state in
    let true_list = [Imp.True] in
    let false_list = [Imp.False] in
    let (sigma_n, sym_sigma_n, ssa_n, path_f, path, props) =
      try
	SymExec.eval_prog program sigma_0 sym_sigma_0 ssa_0 [] [] true_list
      with Failure msg -> Printf.printf "%s" msg; (sigma_0, sym_sigma_0, ssa_0, [], [], false_list)
    in*)
    
    (* Collect concrete paths through the program *)
    let (sigma_n, path) =
      try
	ConcreteExec.eval_prog program sigma_0 []
      with Failure msg -> Printf.printf "%s" msg; (sigma_0, [])
    in
    
    
    path' := path;
    if !path_debug then( 
      print_endline "PRINTING PATH:";
      let path_list = List.map string_of_int !path' in
      let path_str = String.concat ";" path_list in
      prerr_endline ("["^path_str^"]");
    
      print_endline "SLICING PATH...";
    );
    let (prog, inv_prog) = Preprocess.get_progs () in
    let slice = PathSlice.path_slice path prog in

    if !path_debug then( 
      print_endline "ORIGINAL PATH:";
      let path_list = List.map string_of_int !path' in
      let path_str = String.concat ";" path_list in
      prerr_endline ("["^path_str^"]");
      print_endline "SLICED PATH:";
      let slice_list = List.map string_of_int slice in
      let slice_str = String.concat ";" slice_list in
      prerr_endline ("["^slice_str^"]");
 
      print_endline "SSA PHASE...";
    );

    (* Old way *)
    (*let ssa_comhash = GuidedSSA.ssa_path !path' in

    if !path_debug then( 
      print_endline "SYMBOLIC EXECUTION PHASE...");
    let (sym_sigma, path_formula, property) = GuidedSymExec.sym_eval_path slice ssa_comhash in
    let formula_str_list = List.map Imp.bexp_to_str (property::path_formula) in
    let formula_str = String.concat " /\\ " formula_str_list in

    let file = open_out ("tests/iteration-"^(string_of_int !iteration)^".old") in
    let _ = output_string file formula_str in
    let _ = close_out file in


    *)
    let ssa_comhash = GuidedSSA.ssa_path slice in

    if !path_debug then( 
      print_endline "SYMBOLIC EXECUTION PHASE...");
    let (sym_sigma, path_formula, property) = GuidedSymExec.sym_eval_path slice ssa_comhash in
    (*let formula_str_list = List.map Imp.bexp_to_str (property::path_formula) in
    let formula_str = String.concat " /\\ " formula_str_list in

    let file = open_out ("tests/iteration-"^(string_of_int !iteration)^".new") in
    let _ = output_string file formula_str in
    let _ = close_out file in*)
   
    path' := slice;
    let uniqueQ = Util.is_unique !path' upaths in
    if uniqueQ 
    then( 
      let path_list = List.map string_of_int !path' in
      let path_str = String.concat ";" path_list in
      prerr_endline ("UNIQUE: ["^path_str^"]");
      unique_paths_num := !unique_paths_num + 1; 
      let r_index = !GuidedSymExec.r_index in
      let props = [property] in
      Util.add_path upaths !path' sym_sigma path_formula r_index props;
      no_new := 0
    )else
      no_new := !no_new + 1
  done;
    (*;*)
  (*print_endline ("Done collecting paths. #Paths: "^(string_of_int (Util.Pathhash.length upaths)));
  print_endline ("UPaths: "^(string_of_int !unique_paths_num));*)
  prerr_endline ("Done collecting paths. #Paths: "^(string_of_int (Util.Pathhash.length upaths)));
  upaths
