
(*--- 

frontend: A prototype frontend for probabilistic programs

    Copyright (C) 2013  Aleksandar Chakarov <alch4231@colorado.edu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

--*)

open Imp

module LocMap = Map.Make(struct
  type t = loc
  let compare = compare
end)

type symbolic_state = aexp LocMap.t
let empty_symbolic_state: symbolic_state = LocMap.empty

module InverseComhash = Hashtbl.Make(
  struct
    type t = com
    let equal x y = (x = y)
    let hash = Hashtbl.hash
  end
)

let make_inverse_com_hash () = 
  let new_inverse_com_hash : int InverseComhash.t = InverseComhash.create 10000 in
  new_inverse_com_hash

module Comhash = Hashtbl.Make(
  struct
    type t = int
    let equal x y = (x = y)
    let hash = Hashtbl.hash
  end
)

let make_com_hash () = 
  let new_com_hash : com Comhash.t = Comhash.create 10000 in
  new_com_hash

module Branchhash = Hashtbl.Make(
  struct
    type t = int
    let equal x y = (x = y)
    let hash = Hashtbl.hash
  end
)

let make_branch_hash () = 
  let new_branch_hash : bool list Branchhash.t = Branchhash.create 10000 in
  new_branch_hash

let branches = ref (make_branch_hash ())

let branch_lookup (com_id: int) : bool list = 
  try
    Branchhash.find !branches com_id
  with Not_found -> []

let branch_update (com_id: int) (branch_decision: bool) : unit = 
  let branch_found = branch_lookup com_id in
  Branchhash.replace !branches com_id (branch_found@[branch_decision])

let get_next_branch_decision (com_id: int) : bool =
  let branch_found = branch_lookup com_id in
  let branch_decision = List.hd branch_found in
  Branchhash.replace !branches com_id (List.tl branch_found);
  branch_decision


module Simplecomhash = Hashtbl.Make(
  struct
    type t = int
    let equal x y = (x = y)
    let hash = Hashtbl.hash
  end
)

let make_simple_com_hash () = 
  let new_simple_com_hash : simple_com Simplecomhash.t = Simplecomhash.create 10000 in
  new_simple_com_hash

let  listEquals x y =
  if (List.length x) = (List.length y) 
  then  List.for_all2 ( fun a b -> a = b) x y  
  else false 

module Pathhash = Hashtbl.Make(
  struct
    type t = int list
    let equal x y = (listEquals x y)
    let hash = Hashtbl.hash
  end
)

(*let make_upath_hash () =
  let new_path_hash : int Pathhash.t = Pathhash.create 10000 in
  new_path_hash *)

let is_unique (path: int list) (upaths) : bool = 
  not(Pathhash.mem upaths path)

(* i is a placeholder for whatever comes next *)
(*let add_path (upaths) (path: int list) (i: int) = 
  Pathhash.replace upaths path i
*)


let make_upath_hash () = 
  let new_path_hash : (symbolic_state * bexp list * int * bexp list) Pathhash.t = Pathhash.create 10000 in
  new_path_hash

(*
let is_unique (path: int list) (upaths) : bool = 
  not(Pathhash.mem upaths path)
*)

(* old add_path copies over the !current_assign and stores the list for path in upaths) *)
let add_path (upaths) (path: int list) (sym_sigma: symbolic_state) (path_f: bexp list) (r_index: int) (props: bexp list) = 
  Pathhash.replace upaths path (sym_sigma, path_f, r_index, props)

