/*---

volComp: A prototype Volume Computation Tool.

    Copyright (C) 2013  Sriram Sankaranarayanan <srirams@colorado.edu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

---*/

#ifndef D__PROBTYPE__H__
#define D__PROBTYPE__H__

//#define PROBTYPE_USE_NATIVE 1

#ifdef PROBTYPE_USE_NATIVE


typedef double  prob_type_t ;
//const int prec = 40;

static void assignProbProb(prob_type_t & p, prob_type_t q){
  p = q;
}

static void assignProb(prob_type_t & val, double d){
  val = d;
}

static void allocNumberProb(prob_type_t & rVal, double d=0.0){
  rVal = d;
  return;
}

static void deallocNumberProb (prob_type_t & p){
  p = 0;
}

static void addAssign(prob_type_t & p, prob_type_t q){
  p = p + q;
}

static void subAssign(prob_type_t & p, prob_type_t q){
  p = p - q;
}


static void mulAssign(prob_type_t & p, prob_type_t q){
  p = p * q;
}

static void multiplyFloatAssign(prob_type_t & p, double d){
  p = p * d;
}


static void addFloatAssign(prob_type_t & p, double d){
  p = p + d;
}

extern char intlBuf[1024];

static  char* probPrint(prob_type_t p){
  sprintf(intlBuf,"%f",p);
  return intlBuf;
}
#else 
#include <cstdio>
#include <mpfr.h>

typedef mpfr_t prob_type_t ;
const int prec = 40;

static void assignProbProb(prob_type_t & p, prob_type_t q){
  mpfr_set(p,q,MPFR_RNDN);
}


static bool probType_geq(prob_type_t & p, prob_type_t q){
  return (mpfr_cmp(p,q) >= 0);
}

static void assignProb(prob_type_t val, double d){
  mpfr_set_d(val,d,MPFR_RNDN);
}

static void allocNumberProb(prob_type_t rVal, double d=0.0){
  mpfr_init2(rVal,prec);
  mpfr_set_d(rVal, d, MPFR_RNDN);
  return;
}

static void deallocNumberProb (prob_type_t p){
  mpfr_clear(p);
}

static void addAssign(prob_type_t p, prob_type_t q){
  mpfr_add(p, p, q, MPFR_RNDN);
}

static void subAssign(prob_type_t p, prob_type_t q){
  mpfr_sub(p, p, q, MPFR_RNDN);
}


static void mulAssign(prob_type_t p, prob_type_t q){
  mpfr_mul(p, p, q, MPFR_RNDN);
}

static void multiplyFloatAssign(prob_type_t p, double d){
  mpfr_mul_d(p, p, d, MPFR_RNDN);
}


static void addFloatAssign(prob_type_t p, double d){
  mpfr_add_d(p, p, d, MPFR_RNDN);
}

extern char intlBuf[1024];

static  char* probPrint(prob_type_t p){
  mpfr_sprintf(intlBuf,"%.16Rf",p);
  return intlBuf;
}
#endif
#endif
