/*---

volComp: A prototype Volume Computation Tool.

    Copyright (C) 2013  Sriram Sankaranarayanan <srirams@colorado.edu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

---*/

#ifndef D__UNION_FIND_HH__
#define D__UNION_FIND_HH__

#include "myUtils.hh"

/* 
   Partition Implementation for Quick Clustering
*/


typedef struct {
  int nVars;
  int *  parent;
  int *  rank;
} dset_t;


static int getRank(dset_t * d, int i) {
  return d -> rank[i];
}

static int getParent(dset_t * d, int i) {
  return d -> parent[i];
}

static void setParent(dset_t * d, int i, int j){
  d -> parent[i] = j;
}

static void setRank(dset_t * d, int i, int j){
  d -> rank[i]  =j;
}




static dset_t * createDisjointSets( int numVars ){
  dset_t * rSet = new dset_t();
  rSet -> nVars = numVars;
  rSet -> rank = new int[numVars];
  rSet -> parent = new int[numVars];
  
  for (int i = 0; i < numVars; ++i){
    rSet -> rank[i] = 0;
    rSet -> parent[i] = i;
  }
  return rSet;
}

static void deleteDisjointSets(dset_t * d){
  delete(d -> parent);
  delete(d -> rank);
  delete(d);
}


static void link(dset_t * d, int i, int j){
  int rI = getRank(d,i);
  int rJ = getRank(d,j);
  
  if (rI > rJ){
    // Link j to i
    setParent(d,j,i);
  } else {
    // Link i to j
    setParent(d,i,j);
    if (rI == rJ)
      setRank(d,j,rJ+1);
  }
}


static int find (dset_t * d, int i){
  int pI = getParent(d,i);
  if (i == pI) return i;
  int qI = find(d,pI);
  setParent(d,i,qI);
  return qI;
}

static bool doUnion(dset_t * d, int i, int j){
  int qI = find(d,i);
  int qJ = find(d,j);
  if (qI == qJ) return false;
  link(d,qI,qJ);
  return true;
}

static void collectClusters(dset_t * d, map<int, set<int> > & clusterMap){
  int i;
  for (i = 0; i < d -> nVars; ++i){
    int qI = find(d,i); 
    map<int, set<int> >::iterator iT = clusterMap.find(qI);
    if (iT == clusterMap.end()){
      set<int> sI;
      sI.insert(i);
      sI.insert(qI);
      clusterMap.insert( pair<int, set<int> > ( qI, sI));
    } else {
      set<int> & rSet = iT -> second;
      rSet.insert(i);
    }
  }
}


#endif
